// Header file for USB+Ethernet/DMX Generator  ********************************
//
// Simple DMX512(incl. Art-Net) controller for STM32F042F6 (20pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// supports the essential features of Art-Net, the MiniDMX protocol
// plus simple MIDI message and ASCII text based command sets
//
// Author and (c)2017-18 Wolfgang Schemmert. The copyright claim and GPL-3 Licence
// is restricted to this file and the depending files
// "main.c", "usb-fs.c", "dmx.c", "ether.c" and "userinterface.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz 1.0 or 1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

// extern defined in userinterface.c  ************
extern uint8_t gcPushByte, gcSetupMode;
extern void  ASCII_Interpreter(uint8_t);
extern void MIDI_Interpreter(uint8_t,uint8_t,uint8_t,uint8_t);
extern uint8_t Setup_Interpreter(uint8_t);
extern void copyComParams(void);

// extern defined in usb-fs.c  ************
extern uint8_t guPushbyte, gcIgnoreVelo0;
extern uint8_t guDeviceClass, guConfigured;
extern uint8_t guUsbCmdEnable;      //simply to speed up sendCmdB() a little bit
extern void stopUSB(void);
extern void initUSB(void);
extern uint16_t pullUsbRxBuf(void);
extern void sendUsbB(uint8_t);
extern uint32_t pullMidiMsg(void);

// extern defined in ether.c  ************
extern uint8_t geSPI_RXBUF[];
extern uint8_t *geSPI_RXHEAD, *geSPI_RXTAIL, *geSPI_RXBUFTOP;
extern uint8_t geSPI_TXBUF[];
extern uint8_t *geSPI_TXHEAD, *geSPI_TXTAIL, *geSPI_TXBUFTOP;
extern uint8_t geArtnetOn;  //1=default: Artnet data are send to DMX. 0=block

extern void initSPI1(void);
uint8_t exchangeSpiByte(uint8_t);
extern uint8_t initW5500(void);
extern uint8_t openUDP0(void);
extern uint8_t sendEtherByte(uint8_t);
extern uint8_t receiveUDP(void);
extern void writeWiz_4(uint16_t, uint8_t, uint8_t);
extern uint8_t readWiz_4(uint16_t, uint8_t);
extern void writeWiz_5(uint16_t, uint8_t, uint16_t);
extern uint16_t readWiz_5(uint16_t, uint8_t);
extern void writeWiz_7(uint16_t, uint8_t, uint32_t);
extern void writeWiz_Many(uint16_t, uint8_t, uint8_t *, uint8_t);
extern void readWiz_Data(uint16_t, uint8_t);

// extern defined in dmx.c
extern uint8_t gdDMXBUF[DMX_BUFLEN+7], gdFINALMEM[DMX_BUFLEN+7];
extern uint8_t gdFaderBusy, gdMiniDmxOn;
extern uint16_t gdDmxAddr, gdDmxTxIndex, gdFadeTime, gdDmxMax;
extern void startDmxTx(void);
extern void DmxHeader(void);
extern void MiniDMX_Interpreter(void);
extern void fade2Preset(void);
extern void fadeProcess(void);
extern void sendAddrLevel(void);
extern void sendLevel(uint16_t);

// ****************************************

//system control
uint16_t gSYSCOUNT=20, gRED_LEDCOUNT, gGREEN_LEDCOUNT;
uint16_t gDELAYCOUNT=0,gTIMEOUT=0;
uint8_t gDmxEnable, gDmxReset;
uint8_t gMidiMsgState, gMidiStatus, gMidiData1, gMidiData2;
uint8_t gMINIDMX_ON;

#define PAGE17 (0x08004400)
#define PAGE18 (0x08004800)

struct PARAM{
uint32_t VidPid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t Gateway;
uint32_t ArtnetIP;
uint32_t ArtSubnet;
uint32_t ArtUniverse;
uint32_t Channel;
} gComParam;  //10 words

uint16_t gVidShadow, gPidShadow;

void msDelay(uint16_t) ;
void servSystem(void);
void startUsb(void);
void popFlashPage17(void);
void pushFlashPage17(void);
void popPresetPage(uint8_t);
void pushPresetPage(uint8_t);
uint16_t parseStatus(void);
uint8_t parseData(uint8_t);

#ifdef DMX_DEBUG
void initUsart2();
void sendB2(uint8_t);
void sendX8(uint8_t);
void sendX32(uint32_t);
#endif

void sendUsbString(const uint8_t *);
void sendUsbNibble(uint8_t);
void sendUsbX8(uint8_t);
void sendUsbN8(uint8_t);
void sendUsbX16(uint16_t);
void sendUsbX32(uint32_t);
void sendUsbN32(uint32_t);
void sendUsbS32(uint32_t);
void sendUsbS16(uint16_t);

void SysTick_Handler(void);
void USART2_IRQHandler(void);
